@@ -1,5 +1,5 @@
-// Copyright (C) 2010 NICTA (www.nicta.com.au)
-// Copyright (C) 2010 Conrad Sanderson
+// Copyright (C) 2010-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2010-2011 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -316,9 +316,10 @@ template<typename T1, typename T2>
 arma_inline
 arma_warn_unused
 typename T1::elem_type
-as_scalar(const Glue<T1, T2, glue_times>& X)
+as_scalar(const Glue<T1, T2, glue_times>& X, const typename arma_not_cx<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
   if(is_glue_times_diag<T1>::value == false)
     {
